#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>

GLfloat eye[3] = { 0.0, 0.0, 15.0 };
GLfloat dir[3] = { 0.0, 0.0, -1.0 };

void display(void)
{
	GLfloat center[3];
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	center[0] = eye[0] + dir[0];
	center[1] = eye[1] + dir[1];
	center[2] = eye[2] + dir[2];

	gluLookAt(
		eye[0], eye[1], eye[2],
		center[0], center[1], center[2],
		0.0, 1.0,  0.0);	// up
	
	// draw first sphere
	glPushMatrix();
	glTranslatef(3.0, 0.0, 0.0);
	glColor4f(0.7, 0.4, 0.4, 1.0);
	glutWireSphere(2.0, 12, 12);
	glPopMatrix();

	// draw second sphere
	glPushMatrix();
	glTranslatef(-3.0, 0.0, 0.0);
	glColor4f(0.4, 0.4, 0.7, 1.0);
	glutWireSphere(2.0, 12, 12);
	glPopMatrix();
	
	glFlush();
	glutSwapBuffers();
}

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (double)w / (double)h, 0.1, 100.0);

	glMatrixMode(GL_MODELVIEW);
}

void keyboard(unsigned char key, int x, int y)
{
	GLfloat a[3];
	GLfloat angle = 5.0 * M_PI / 180.0;
	
	switch (key)
	{
		case 'w':
			eye[0] += dir[0];
			eye[1] += dir[1];
			eye[2] += dir[2];
			glutPostRedisplay();
			break;
		case 's':
			eye[0] -= dir[0];
			eye[1] -= dir[1];
			eye[2] -= dir[2];
			glutPostRedisplay();
			break;
		case 'a':
			a[0] =  dir[0] * cos(angle) + dir[2] * sin(angle);
			a[1] =  dir[1];
			a[2] = -dir[0] * sin(angle) + dir[2] * cos(angle);
			dir[0] = a[0];
			dir[1] = a[1];
			dir[2] = a[2];
			glutPostRedisplay();
			break;
		case 'd':
			a[0] =  dir[0] * cos(-angle) + dir[2] * sin(-angle);
			a[1] =  dir[1];
			a[2] = -dir[0] * sin(-angle) + dir[2] * cos(-angle);
			dir[0] = a[0];
			dir[1] = a[1];
			dir[2] = a[2];
			glutPostRedisplay();
			break;
		case 27: exit(0);
	}
}

int main(int argc, char ** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH | GLUT_ALPHA);
	glutInitWindowPosition(0, 0);
	glutInitWindowSize(300, 300);
	glutCreateWindow("OpenGL Demo 4");
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glEnable(GL_DEPTH_TEST);
	
	glutMainLoop();
	return 0;
}
